@@ -1,4 +1,4 @@
-from collections import namedtuple
+from collections import deque, namedtuple
 from functools import partial
 from itertools import groupby
 from math import ceil
@@ -19,6 +19,7 @@ def stream_sqlite(sqlite_chunks):
     index_interior_header = Struct('>HHHBL')
     freelist_trunk_header = Struct('>LL')
 
+    master_row_constructor = namedtuple('MasterRow', ('type', 'name', 'tbl_name', 'rootpage', 'sql'))
     column_constructor = namedtuple('Column', ('cid', 'name', 'type', 'notnull', 'dflt_value', 'pk'))
 
     def get_byte_reader(iterable):
@@ -143,86 +144,119 @@ def stream_sqlite(sqlite_chunks):
 
         def process_table_page(table_name, table_info, row_constructor, page_bytes, page_reader):
 
-            def get_master_table(master_cells):
+            def yield_leaf_table_rows(pointers):
 
-                def table_info_and_row_constructor(cur, table_name, sql):
-                    cur.execute(sql)
-                    cur.execute("PRAGMA table_info('" + table_name.replace("'","''") + "');")
-                    columns = cur.fetchall()
-                    row_constructor = namedtuple('Row', (column[1] for column in columns))
-                    return tuple(column_constructor(*column) for column in columns), row_constructor
+                def get_payload_size_on_leaf(p):
+                    u = len(page_bytes)
+                    x = u - 35
+                    m = (32 * (u - 12) // 255 - 23)
+                    k = (m + (p - m) % (u - 4))
 
-                with connect(':memory:') as con:
-                    cur = con.cursor()
-
-                    return tuple(
-                        (
-                            cell[0],  # table or index
-                            cell[1],  # table name
-                            table_info_and_row_constructor(cur, cell[1], cell[4]) if cell[0] == 'table' else (None, None),
-                            cell[3],  # root page
-                        )
-                        for cell in master_cells
-                        if cell[0] in ('table', 'index')
+                    return (
+                        p if p <= x else \
+                        k if k <= x else \
+                        m
                     )
 
-            def yield_leaf_table_cells(pointers):
-
-                def serial_types(header_remaining, cell_varint_reader):
-                    while header_remaining:
-                        h, v_size = cell_varint_reader()
-                        yield h
-                        header_remaining -= v_size
-
                 for pointer, in pointers:
                     cell_num_reader, cell_varint_reader = get_chunk_readers(page_bytes, pointer)
 
                     payload_size, _ = cell_varint_reader()
                     rowid, _ = cell_varint_reader()
 
-                    header_remaining, header_varint_size = cell_varint_reader()
-                    header_remaining -= header_varint_size
+                    payload_size_on_leaf = get_payload_size_on_leaf(payload_size)
 
-                    yield tuple(
-                        parser(cell_num_reader(length))
-                        for serial_type in tuple(serial_types(header_remaining, cell_varint_reader))
-                        for (length, parser) in (
-                            (0, lambda _: None) if serial_type == 0 else \
-                            (1, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 1 else \
-                            (2, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 2 else \
-                            (3, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 3 else \
-                            (4, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 4 else \
-                            (6, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 5 else \
-                            (8, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 6 else \
-                            (8, lambda raw: double.unpack(raw)[0]) if serial_type == 7 else \
-                            (0, lambda _: 0) if serial_type == 8 else \
-                            (0, lambda _: 1) if serial_type == 9 else \
-                            (int((serial_type - 12)/2), lambda raw: raw) if serial_type % 2 == 0 else \
-                            (int((serial_type - 13)/2), lambda raw: raw.decode()) if serial_type % 2 == 1 else \
-                            (None, None),
-                        )
+                    if payload_size_on_leaf == payload_size:
+                        yield read_table_row(cell_num_reader, cell_varint_reader)
+
+                    else:
+                        payload_first_chunk = cell_num_reader(payload_size_on_leaf)
+                        overflow_page, = unsigned_long.unpack(cell_num_reader(4))
+                        payload_chunks = deque()
+                        payload_chunks.append(payload_first_chunk)
+                        payload_remainder = payload_size - payload_size_on_leaf
+
+                        yield from process_if_buffered_or_remember(partial(
+                            process_overflow_leaf_table_page,
+                            table_name, table_info, row_constructor, payload_chunks, payload_remainder
+                        ), overflow_page)
+
+            def read_table_row(cell_num_reader, cell_varint_reader):
+
+                def serial_types(header_remaining, cell_varint_reader):
+                    while header_remaining:
+                        h, v_size = cell_varint_reader()
+                        yield h
+                        header_remaining -= v_size
+
+                header_remaining, header_varint_size = cell_varint_reader()
+                header_remaining -= header_varint_size
+
+                return row_constructor(*tuple(
+                    parser(cell_num_reader(length))
+                    for serial_type in tuple(serial_types(header_remaining, cell_varint_reader))
+                    for (length, parser) in (
+                        (0, lambda _: None) if serial_type == 0 else \
+                        (1, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 1 else \
+                        (2, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 2 else \
+                        (3, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 3 else \
+                        (4, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 4 else \
+                        (6, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 5 else \
+                        (8, lambda raw: int.from_bytes(raw, byteorder='big', signed=True)) if serial_type == 6 else \
+                        (8, lambda raw: double.unpack(raw)[0]) if serial_type == 7 else \
+                        (0, lambda _: 0) if serial_type == 8 else \
+                        (0, lambda _: 1) if serial_type == 9 else \
+                        (int((serial_type - 12)/2), lambda raw: raw) if serial_type % 2 == 0 else \
+                        (int((serial_type - 13)/2), lambda raw: raw.decode()) if serial_type % 2 == 1 else \
+                        (None, None),
                     )
+                ))
 
             def process_table_leaf_master():
-                _, num_cells, _, _ = \
-                    table_leaf_header.unpack(page_reader(7))
 
-                pointers = unsigned_short.iter_unpack(page_reader(num_cells * 2))
+                def table_info_and_row_constructor(cur, master_row):
+                    cur.execute(master_row.sql)
+                    cur.execute("PRAGMA table_info('" + master_row.name.replace("'","''") + "');")
+                    columns = cur.fetchall()
+                    row_constructor = namedtuple('Row', (column[1] for column in columns))
+                    return tuple(column_constructor(*column) for column in columns), row_constructor
 
-                for table_or_index, table_name, (table_info, row_constructor), root_page in get_master_table(yield_leaf_table_cells(pointers)):
-                    yield from (
-                        process_if_buffered_or_remember(partial(process_table_page, table_name, table_info, row_constructor), root_page) if table_or_index == 'table' else \
-                        process_if_buffered_or_remember(process_index_page, root_page)
-                    )
+                _, num_cells, _, _ = table_leaf_header.unpack(page_reader(7))
+
+                with connect(':memory:') as con:
+                    cur = con.cursor()
+
+                    pointers = unsigned_short.iter_unpack(page_reader(num_cells * 2))
+
+                    for master_row in yield_leaf_table_rows(pointers):
+                        yield from (
+                            process_if_buffered_or_remember(partial(process_table_page, master_row.name, *table_info_and_row_constructor(cur, master_row)), master_row.rootpage) if master_row.type == 'table' else \
+                            process_if_buffered_or_remember(process_index_page, master_row.rootpage) if master_row.type == 'index' else \
+                            ()
+                        )
 
             def process_table_leaf_non_master():
-                _, num_cells, _, _ = \
-                    table_leaf_header.unpack(page_reader(7))
+                _, num_cells, _, _ = table_leaf_header.unpack(page_reader(7))
 
                 pointers = unsigned_short.iter_unpack(page_reader(num_cells * 2))
 
-                for cell in yield_leaf_table_cells(pointers):
-                    yield table_name, table_info, row_constructor(*cell)
+                for row in yield_leaf_table_rows(pointers):
+                    yield table_name, table_info, row
+
+            def process_overflow_leaf_table_page(table_name, table_info, row_constructor, payload_chunks, payload_remainder, page_bytes, page_reader):
+                next_overflow_page, = unsigned_long.unpack(page_reader(4))
+                num_this_page = min(payload_remainder, len(page_bytes) - 4)
+                payload_remainder -= num_this_page
+                payload_chunks.append(page_reader(num_this_page))
+
+                if not next_overflow_page:
+                    yield table_name, table_info, read_table_row(*get_chunk_readers(b''.join(payload_chunks)))
+
+                else:
+                    yield from process_if_buffered_or_remember(partial(
+                        process_overflow_leaf_table_page,
+                        table_name, table_info, row_constructor, payload_chunks, payload_remainder
+                    ), next_overflow_page)
 
             def process_table_interior():
                 _, num_cells, _, _, right_most_pointer = \
@@ -291,7 +325,7 @@ def stream_sqlite(sqlite_chunks):
             else:
                 yield from process(page_bytes, page_reader)
 
-        page_processors[1] = partial(process_table_page, 'sqlite_schema', (), column_constructor)
+        page_processors[1] = partial(process_table_page, 'sqlite_schema', (), master_row_constructor)
 
         if first_freelist_trunk_page:
             page_processors[first_freelist_trunk_page] = process_freelist_trunk_page
